@@ -8,15 +8,22 @@
 	If we officially support Windows again, we should have this reworked to really cope with Windows paths, too.
 */
 
-#include "config.h"
-#include "mpg123.h"
+#include "mpg123app.h"
 #include "getlopt.h" /* for loptind */
-#include "debug.h"
 #include "term.h" /* for term_restore */
 #include "playlist.h"
+#include "httpget.h"
 
 #include <time.h>
 
+#ifdef HAVE_RANDOM
+#define RAND random
+#define SRAND srandom
+#else
+#define RAND rand
+#define SRAND srand
+#endif
+
 /* increase linebuf in blocks of ... bytes */
 #define LINEBUF_STEP 100
 
@@ -35,7 +42,6 @@ playlist_struct pl;
 
 int add_next_file (int argc, char *argv[]);
 void shuffle_playlist();
-void print_playlist();
 void init_playlist();
 int add_copy_to_playlist(char* new_entry);
 int add_to_playlist(char* new_entry, char freeit);
@@ -53,13 +59,23 @@ void prepare_playlist(int argc, char** argv)
 	if(param.verbose > 1)
 	{
 		fprintf(stderr, "\nplaylist in normal order:\n");
-		print_playlist();
+		print_playlist(stderr, 0);
 		fprintf(stderr, "\n");
 	}
 	if(param.shuffle == 1) shuffle_playlist();
 	/* Don't need these anymore, we have copies! */
-	free_stringbuf(&pl.linebuf);
-	free_stringbuf(&pl.dir);
+	mpg123_free_string(&pl.linebuf);
+	mpg123_free_string(&pl.dir);
+}
+
+/* Return a random number >= 0 and < n */
+static size_t rando(size_t n)
+{
+	long ran;
+	long limit = RAND_MAX - (RAND_MAX % (long)n);
+	if(n<2) return 0; /* Better settle that here than in an endless loop... */
+	do{ ran = RAND(); }while( ran >= limit );
+	return (size_t)(ran%n);
 }
 
 char *get_next_file()
@@ -71,14 +87,26 @@ char *get_next_file()
 	/* normal order, just pick next thing */
 	if(param.shuffle < 2)
 	{
-		if(pl.pos < pl.fill) newfile = pl.list[pl.pos].url;
-		else newfile = NULL;
-		++pl.pos;
+		do
+		{
+			if(pl.pos < pl.fill) newfile = pl.list[pl.pos].url;
+			else newfile = NULL;
+			/* if we have rounds left, decrease loop, else reinit loop because it's a new track */
+			if(pl.loop > 0) --pl.loop; /* loop for current track... */
+			if(pl.loop == 0)
+			{
+				pl.loop = param.loop;
+				++pl.pos;
+			}
+		} while(pl.loop == 0 && newfile != NULL);
+	}
+	else
+	{	/* Randomly select files, with repeating... but keep track of current track for playlist printing. */
+		pl.pos = rando(pl.fill)+1;
+		newfile = pl.list[ pl.pos-1 ].url;
 	}
-	/* randomly select files, with repeating */
-	else newfile = pl.list[ (size_t) rand() % pl.fill ].url;
 
-	return newfile;
+	return newfile; /* "-" is STDOUT, "" is dumb, NULL is nothing */
 }
 
 /* It doesn't really matter on program exit, but anyway...
@@ -99,14 +127,14 @@ void free_playlist()
 		pl.size = 0;
 		debug("free()d the playlist");
 	}
-	free_stringbuf(&pl.linebuf);
-	free_stringbuf(&pl.dir);
+	mpg123_free_string(&pl.linebuf);
+	mpg123_free_string(&pl.dir);
 }
 
 /* the constructor... */
 void init_playlist()
 {
-	srand(time(NULL));
+	SRAND(time(NULL));
 	pl.file = NULL;
 	pl.entry = 0;
 	pl.size = 0;
@@ -114,9 +142,10 @@ void init_playlist()
 	pl.pos = 0;
 	pl.list = NULL;
 	pl.alloc_step = 10;
-	init_stringbuf(&pl.dir);
-	init_stringbuf(&pl.linebuf);
+	mpg123_init_string(&pl.dir);
+	mpg123_init_string(&pl.linebuf);
 	pl.type = UNKNOWN;
+	pl.loop = param.loop;
 }
 
 /*
@@ -139,7 +168,7 @@ int add_next_file (int argc, char *argv[])
 		if ((slashpos=strrchr(param.listname, '/')))
 		{
 			/* up to and including /, with space for \0 */
-			if(resize_stringbuf(&pl.dir, 2 + slashpos - param.listname))
+			if(mpg123_resize_string(&pl.dir, 2 + slashpos - param.listname))
 			{
 				memcpy(pl.dir.p, param.listname, pl.dir.size-1);
 				pl.dir.p[pl.dir.size-1] = 0;
@@ -167,18 +196,22 @@ int add_next_file (int argc, char *argv[])
 			else if (!strncmp(param.listname, "http://", 7))
 			{
 				int fd;
-				char *listmime = NULL;
-				fd = http_open(param.listname, &listmime);
-				debug1("listmime: %p", (void*) listmime);
-				if(listmime != NULL)
+				struct httpdata htd;
+				httpdata_init(&htd);
+				fd = http_open(param.listname, &htd);
+				debug1("htd.content_type.p: %p", (void*) htd.content_type.p);
+				if(htd.content_type.p != NULL)
 				{
-					debug1("listmime value: %s", listmime);
-					if(!strcmp("audio/x-mpegurl", listmime))	pl.type = M3U;
-					else if(!strcmp("audio/x-scpls", listmime) || !strcmp("application/pls", listmime))	pl.type = PLS;
+					int mimi;
+					debug1("htd.content_type.p value: %s", htd.content_type.p);
+					mimi = debunk_mime(htd.content_type.p);
+
+					if(mimi & IS_M3U) pl.type = M3U;
+					else if(mimi & IS_PLS)	pl.type = PLS;
 					else
 					{
 						if(fd >= 0) close(fd);
-						if(!strcmp("audio/mpeg", listmime) || !strcmp("audio/x-mpeg", listmime))
+						if(mimi & IS_FILE)
 						{
 							pl.type = NO_LIST;
 							if(param.listentry < 0)
@@ -193,10 +226,10 @@ int add_next_file (int argc, char *argv[])
 								return 1;
 							}
 						}
-						fprintf(stderr, "Error: unknown playlist MIME type %s; maybe "PACKAGE_NAME" can support it in future if you report this to the maintainer.\n", listmime);
+						fprintf(stderr, "Error: unknown playlist MIME type %s; maybe "PACKAGE_NAME" can support it in future if you report this to the maintainer.\n", htd.content_type.p);
 						fd = -1;
 					}
-					free(listmime);
+					httpdata_reset(&htd);
 				}
 				if(fd < 0)
 				{
@@ -238,7 +271,7 @@ int add_next_file (int argc, char *argv[])
 				/* have is the length of the string read, without the closing \0 */
 				if(pl.linebuf.size <= have+1)
 				{
-					if(!resize_stringbuf(&pl.linebuf, pl.linebuf.size+LINEBUF_STEP))
+					if(!mpg123_resize_string(&pl.linebuf, pl.linebuf.size+LINEBUF_STEP))
 					{
 						error("cannot increase line buffer");
 						break;
@@ -258,7 +291,6 @@ int add_next_file (int argc, char *argv[])
 			} while(have && pl.linebuf.p[have-1] != '\r' && pl.linebuf.p[have-1] != '\n');
 			if(have)
 			{
-				size_t i;
 				pl.linebuf.p[strcspn(pl.linebuf.p, "\t\n\r")] = '\0';
 				/* a bit of fuzzyness */
 				if(firstline)
@@ -306,11 +338,14 @@ int add_next_file (int argc, char *argv[])
 					firstline = 0;
 				}
 				#if !defined(WIN32)
+				{
+				size_t i;
 				/* convert \ to / (from MS-like directory format) */
 				for (i=0;pl.linebuf.p[i]!='\0';i++)
 				{
 					if (pl.linebuf.p[i] == '\\')	pl.linebuf.p[i] = '/';
 				}
+				}
 				#endif
 				if (pl.linebuf.p[0]=='\0') continue; /* skip empty lines... */
 				if (((pl.type == M3U) && (pl.linebuf.p[0]=='#')))
@@ -370,7 +405,7 @@ int add_next_file (int argc, char *argv[])
 					need = pl.dir.size + strlen(pl.linebuf.p+line_offset);
 					if(pl.linebuf.size < need)
 					{
-						if(!resize_stringbuf(&pl.linebuf, need))
+						if(!mpg123_resize_string(&pl.linebuf, need))
 						{
 							error("unable to enlarge linebuf for appending path! skipping");
 							continue;
@@ -413,26 +448,12 @@ void shuffle_playlist()
 	size_t rannum;
 	if(pl.fill >= 2)
 	{
+		/* Refer to bug 1777621 for discussion on that.
+		   It's Durstenfeld... */
 		for (loop = 0; loop < pl.fill; loop++)
 		{
 			struct listitem tmp;
-			/*
-				rand gives integer 0 <= RAND_MAX
-				dividing this by (fill-1)*4 and taking the rest gives something 0 <= x < (fill-1)*4
-				now diving x by 4 gives 0 <= y < fill-1
-				then adding 1 if y >= loop index... makes 1 <= z <= fill-1
-				if y not >= loop index that means... what?
-
-				rannum = (rand() % (fill * 4 - 4)) / 4;
-				rannum += (rannum >= loop);
-
-				why not simply
-
-				rannum = ( rand() % fill);
-				
-				That directly results in a random number in the allowed range. I'm using this now until someone convinces me of the numerical benefits of the other.
-			*/
-			rannum = (size_t) rand() % pl.fill;
+			rannum = loop + rando(pl.fill-loop);
 			/*
 				Small test on your binary operation skills (^ is XOR):
 				a = b^(a^b)
@@ -455,18 +476,25 @@ void shuffle_playlist()
 	{
 		/* print them */
 		fprintf(stderr, "\nshuffled playlist:\n");
-		print_playlist();
+		print_playlist(stderr, 0);
 		fprintf(stderr, "\n");
 	}
 }
 
-void print_playlist()
+void print_playlist(FILE* out, int showpos)
 {
 	size_t loop;
 	for (loop = 0; loop < pl.fill; loop++)
-	fprintf(stderr, "%s\n", pl.list[loop].url);
+	{
+		char *pre = "";
+		if(showpos)
+		pre = (pl.pos>0 && loop==pl.pos-1) ? "> " : "  ";
+
+		fprintf(out, "%s%s\n", pre, pl.list[loop].url);
+	}
 }
 
+
 int add_copy_to_playlist(char* new_entry)
 {
 	char* cop;
@@ -485,7 +513,7 @@ int add_to_playlist(char* new_entry, char freeit)
 	{
 		struct listitem* tmp = NULL;
 		/* enlarge the list */
-		tmp = (struct listitem*) realloc(pl.list, (pl.size + pl.alloc_step) * sizeof(struct listitem));
+		tmp = (struct listitem*) safe_realloc(pl.list, (pl.size + pl.alloc_step) * sizeof(struct listitem));
 		if(!tmp)
 		{
 			error("unable to allocate more memory for playlist");
