@@ -2,7 +2,7 @@
 	term: terminal control
 
 	copyright ?-2006 by the mpg123 project - free software under the terms of the LGPL 2.1
-	see COPYING and AUTHORS files in distribution or http://mpg123.de
+	see COPYING and AUTHORS files in distribution or http://mpg123.org
 	initially written by Michael Hipp
 */
 
@@ -44,6 +44,14 @@
 /* #define FINE_REWIND_KEY ';' */
 /* #define FINE_FORWARD_KEY ':' */
 
+#define VOL_UP_KEY '+'
+#define VOL_DOWN_KEY '-'
+#define VERBOSE_KEY 'v'
+#define RVA_KEY 'r'
+#define PLAYLIST_KEY 'l'
+#define PREV_KEY 'd'
+#define MPEG_KEY 'm'
+#define TAG_KEY  't'
 /* this counts as "undocumented" and can disappear */
 #define FRAME_INDEX_KEY 'i'
 
@@ -52,7 +60,7 @@
 #define EMPTY_STRING	"        \b\b\b\b\b\b\b\b"
 
 void term_init(void);
-long term_control(struct frame *fr, struct audio_info_struct *ai);
+off_t term_control(mpg123_handle *mh);
 void term_restore(void);
 
 #endif
